from django.core.exceptions import ValidationError
from django.forms.fields import Field


class BaseOperator(object):
    
    @classmethod
    def create(cls, *args):
        instance = cls(*args)
        instance.required = any(each.required for each in instance.operands)
        return instance
    
    def __init__(self, *args):
        self.operands = args

    def __len__(self):
        size = lambda x: len(x) if isinstance(x, BaseOperator) else 1
        return sum(size(each) for each in self.operands)
        
    def __str__(self):
        func = lambda x,y: "%s %s %s" %(
            x.attrname if hasattr(x, 'attrname') else x, 
            self.__class__.__name__ ,
            y.attrname if hasattr(y, 'attrname') else y,
        )
        return "( %s )" % reduce(func, self.operands)

    def raise_exception(self, subject):
        text = 'Expected logic: %s' %self
        exception = ValidationError(text)
        exception.subject = subject
        raise exception
    
    def is_valid_data(self, obj, validated_data, cleaned_data):
        if isinstance(obj, Field):
            if cleaned_data.get(obj.attrname):
                return validated_data.get(obj.attrname)
            return cleaned_data.get(obj.attrname)
        else:
            return obj.is_valid(validated_data, cleaned_data)
        

class OR(BaseOperator):
    def is_valid(self, validated_data, cleaned_data):
        for each in self.operands:
            if self.is_valid_data(each, validated_data, cleaned_data):
                return True
        return False


class AND(BaseOperator): 
    def is_valid(self, validated_data, cleaned_data):
        validated = 0
        for each in self.operands:
            if not self.is_valid_data(each, validated_data, cleaned_data):
                if self.required:
                    self.raise_exception(subject=each.attrname)
            else:
                validated += 1
        if 0 < validated < len(self.operands):
            self.raise_exception(subject=None)
        return True
    


try:
    from .fields import *
except ImportError:
    import sys
    sys.stdout.write("Warning: Could not find the GEOS library.\n")
    from django.forms import *
    
from .mixins import QueryFormMixin
from .operators import AND, OR


class QueryModelForm(ModelForm, QueryFormMixin):
    def __init__(self, *args, **kwargs):
        super(QueryModelForm, self).__init__(*args, **kwargs)
        self.set_meta()
        choice_fields = (ModelChoiceField, ModelMultipleChoiceField)
        for name,field in self.fields:
            if not isinstance(field, choice_fields):
                field.required = False
            elif name in self._meta.required:
                field.required = True
            else:
                field.required = False
                      
    def full_clean(self):
        super(QueryModelForm, self).full_clean()
        self.clean_extralogic()


class QueryForm(Form, QueryFormMixin):    
    def __init__(self, *args, **kwargs):
        super(QueryForm, self).__init__(*args, **kwargs)
        self.set_meta()
    
    def full_clean(self):
        super(QueryForm, self).full_clean()
        self.clean_extralogic()



#!/usr/bin/env python
# encoding: utf-8
from django.test import TestCase
from django.contrib.auth.models import User
from django.test.client import Client, RequestFactory
from django.http import HttpResponse
from .forms import QueryForm, QueryModelForm, Field
from .operators import AND, OR, BaseOperator
from .decorators import validate


A = Field(required=False)
B = Field(required=False)
C = Field(required=False)
D = Field(required=False)
E = Field(required=False)
F = Field(required=False)
            

class QueryModelFormTestCase(TestCase):
    
    def test_validate_decorator(self):
        
        class Form(QueryModelForm):
            class Meta:
                model = User
                fields = ('username', 'email')
                extralogic = [AND('username', 'email'),]
        
        def view(request, pk):
            return HttpResponse('success')
            
        def another_view(request, id):
            return HttpResponse('success')
                
        user = User.objects.create(username='user', email='user@email')
        factory = RequestFactory()
        decorator = validate(Form)
        
        request = factory.get('', {'username':1})
        response = decorator(view)(request, pk=user.pk)
        self.assertNotEqual(response.content, 'success')
        self.assertEqual(request.form.instance, user)
        
        request = factory.get('', {'username':'user', 'email':'1@email.com'})
        response = decorator(view)(request, pk=user.pk)
        self.assertEqual(response.content, 'success')
        self.assertEqual(request.form.instance, user)
        
        request = factory.get('')
        response = decorator(view)(request, pk=user.pk)
        self.assertNotEqual(response.content, 'success')
        self.assertEqual(request.form.instance, user)
        
        decorator = validate(Form, add_instance_using='id')
        request = factory.get('', {})
        response = decorator(another_view)(request, id=user.pk)
        self.assertNotEqual(response.content, 'success')
        self.assertEqual(request.form.instance, user)
        
        decorator = validate(Form, add_instance_using='id')
        request = factory.get('', {'username':1})
        response = decorator(another_view)(request, pk=user.pk)
        self.assertNotEqual(response.content, 'success')
        self.assertNotEqual(request.form.instance, user)
        

class QueryFormTestCase(TestCase):

    def test_BoundingBoxField(self):
        try:
            from .forms import BoundingBoxField
        except ImportError:
            return
            
        class Form(QueryForm):
            box = BoundingBoxField()
        
        lat0, lng0 = 40.0, -8.1 
        lat1, lng1 = 41.0, -9.1   
        f = Form({
            'box_0': '%s,%s'%(lng0,lat0),
            'box_1': '%s,%s'%(lng1,lat1),
        })
        self.assertTrue(f.is_valid())
        self.assertTrue('box' in f.cleaned_data)
        polygon1 = f.cleaned_data['box']
        self.assertEqual(polygon1.num_points, 5)
        self.assertEqual(polygon1, f.parameters['box'])
        
        class Form(QueryForm):
            box = BoundingBoxField(latitude_first=True)
            
            class Meta:
                lookups = {
                    'box': 'point__contained'
                }
            
        f = Form({
            'box_0': '%s,%s'%(lat0,lng0),
            'box_1': '%s,%s'%(lat1,lng1),
        })
        self.assertTrue(f.is_valid())
        self.assertTrue('box' in f.cleaned_data)
        polygon2 = f.cleaned_data['box']
        self.assertEqual(polygon2.num_points, 5)
        self.assertEqual(polygon2, f.parameters['point__contained'])
        self.assertEqual(polygon1, polygon2)
        
    def test_CoordsField(self):
        try:
            from .forms import CoordsField
        except ImportError:
            return
            
        lat, lng = 40.0, -8.1
        
        def assert_form(lat, lng, form):
            self.assertTrue(form.is_valid())
            self.assertTrue('coords' in form.cleaned_data)
            coords = form.cleaned_data['coords'].coords
            self.assertEqual(coords[0], lng)
            self.assertEqual(coords[1], lat)
            self.assertEqual(form.cleaned_data['coords'], form.parameters['coords'])
            
        class Form(QueryForm):
            coords = CoordsField()
        
        f = Form({'coords': '%s,%s'%(lng,lat)})
        assert_form(lat=lat, lng=lng, form=f)

        class Form(QueryForm):
            coords = CoordsField(latitude_first=True)
        
        f = Form({'coords': '%s,%s'%(lat,lng)})
        assert_form(lat=lat, lng=lng, form=f)

    def test_CircleField(self):
        try:
            from .forms import CircleField
        except ImportError:
            return
        
        class Form(QueryForm):
            center = CircleField()
        
        lat, lng = 40.0, -8.1
        f = Form({'center': '%s,%s'%(lng,lat)})
        self.assertTrue(f.is_valid())
        self.assertTrue('center' in f.cleaned_data)
        self.assertTrue(isinstance(f.cleaned_data['center'], tuple))
        coords = f.cleaned_data['center'][0].coords
        self.assertEqual(coords[0], lng)
        self.assertEqual(coords[1], lat)
        self.assertEqual(f.cleaned_data['center'], f.parameters['center'])

    def test_validate_decorator(self):
        
        class Form(QueryForm):
            a, b = A, B
    
            class Meta:
                extralogic = [AND('a', 'b'),]
        
        def view(request):
            return HttpResponse('success')
        
        factory = RequestFactory()
        decorator = validate(Form)
        request = factory.get('', {'a':1})
        wrapper = decorator(view)
        self.assertEqual(wrapper.__name__, view.__name__)
        self.assertNotEqual(wrapper(request).content, 'success')
        request = factory.get('', {'a':1, 'b':1})
        self.assertEqual(wrapper(request).content, 'success')
        request = factory.get('')
        self.assertEqual(wrapper(request).content, 'success')
    
    def test_lookups(self):
        
        class Form(QueryForm):
            a, b, c = A, B, C
    
            class Meta:
                lookups = {
                    'a': 'a__contains',
                    'b': 'b__id',
                }
            
        f = Form({'a': 1, 'b':1, 'c':1})
        lookups = f._meta.lookups.values()
        self.assertEqual(len(lookups), len(f.fields))
        self.assertTrue('a__contains' in lookups)
        self.assertEqual(f._meta.lookups['a'], 'a__contains')
        self.assertTrue('b__id' in lookups)
        self.assertEqual(f._meta.lookups['b'], 'b__id')
        self.assertTrue('c' in lookups)
        self.assertEqual(f._meta.lookups['c'], 'c')
        self.assertFalse('c__year' in lookups)
    
    def test_multifield_lookups(self):
        
        class Form(QueryForm):
            a, b = A, B
    
            class Meta:
                multifield_lookups = {
                    ('a', 'b'): lambda a,b: {'a__range': (a-b, a)}
                }
                
        f = Form({'a':2, 'b':1})
        lookups = f._meta.multifield_lookups
        self.assertTrue(('a', 'b') in lookups)
        self.assertTrue(f.is_valid())
        self.assertTrue('a__range' in f.parameters)
        self.assertTrue('a' in f.validated_data)
        self.assertTrue('b' in f.validated_data)
        self.assertEqual(f.parameters['a__range'], (1, 2))
        
    def test_parameters(self):
        
        class Form(QueryForm):
            a, b, c = A, B, C
    
            class Meta:
                lookups = {
                    'a': 'a__contains',
                    'b': 'b__id',
                    'c': 'c__year',
                }
        
        f = Form({'a': 1, 'b':1, 'c':1})
        self.assertTrue(f.is_valid())
        for fieldname,lookup in f._meta.lookups.items():
            self.assertTrue(f.parameters.get(lookup, None))
            self.assertEqual(f.cleaned_data[fieldname], f.parameters[lookup])
        f = Form({'a': 1})
        self.assertTrue(f.is_valid())
        self.assertTrue('a__contains' in f.parameters)
        self.assertTrue('a' in f.cleaned_data)
        self.assertFalse('b__id' in f.parameters)
        self.assertTrue('b' in f.cleaned_data)
        
    def test_default_data(self):
        class Form(QueryForm):
            a = Field(required=False, initial=1)
        
        f = Form({})
        self.assertTrue(f.is_valid())
        self.assertTrue('a' in f.validated_data)
        self.assertTrue('a' in f.parameters)
        self.assertEqual(f.validated_data['a'], f.fields['a'].initial)
        self.assertEqual(f.validated_data['a'], f.parameters['a'])
        f = Form({'a': 2})
        self.assertTrue(f.is_valid())
        self.assertTrue('a' in f.validated_data)
        self.assertTrue('a' in f.parameters)
        self.assertNotEqual(f.validated_data['a'], f.fields['a'].initial)
        self.assertEqual(f.validated_data['a'], f.parameters['a'])
        
    def test_no_defaults(self):
        
        class Form(QueryForm):
            a = Field(required=False, initial=1)
            
            class Meta:
                no_defaults = True
        
        f = Form({})
        self.assertTrue(f.is_valid())
        self.assertFalse('a' in f.validated_data)
        self.assertFalse('a' in f.parameters)
    
    def test_is_valid_with_or(self):
        
        class Form(QueryForm):
            a, b, c = A, B, C
    
            class Meta:
                extralogic = [
                    OR('a', 'b', 'c'),
                ]
        
        f = Form({})
        self.assertTrue(f.is_valid())
        f = Form({'a':1})
        self.assertTrue(f.is_valid())
        f = Form({'b':1})
        self.assertTrue(f.is_valid())
        f = Form({'c':1})
        self.assertTrue(f.is_valid())
        f = Form({'a':1, 'b':1})
        self.assertTrue(f.is_valid())
        f = Form({'a':1, 'b':1, 'c':1})
        self.assertTrue(f.is_valid())
    
    def test_is_valid_with_and(self):
        
        class Form(QueryForm):
            a, b, c = A, B, C
    
            class Meta:
                extralogic = [
                    AND('a', 'b', 'c'),
                ]
        
        f = Form({})
        self.assertTrue(f.is_valid())
        f = Form({'a':1})
        self.assertFalse(f.is_valid())
        f = Form({'a':1, 'b':1})
        self.assertFalse(f.is_valid())
        f = Form({'a':1, 'b':1, 'c':1})
        self.assertTrue(f.is_valid())
        
    def test_extralogic(self):
        
        class Form(QueryForm):
            a, b, c, d, e, f = A, B, C, D, E, F
    
            class Meta:
                extralogic = [
                    AND('b', 'c', 'd'),
                    OR('e', 'a', 'f'),
                    AND('c', OR('d', 'e'), 'a'),
                    OR(AND('b', 'c', 'd'), 'e'),
                ]
        
        form = Form()
        logic = form._meta.extralogic
        assert len(logic) == 4
        
        assert isinstance(logic[0], AND)
        assert isinstance(logic[0].operands[0], Field)
        assert isinstance(logic[0].operands[1], Field)
        assert isinstance(logic[0].operands[2], Field)
        assert str(logic[0]) == "( b AND c AND d )"
        assert len(logic[0]) == 3
        
        assert isinstance(logic[1], OR)
        assert isinstance(logic[1].operands[0], Field)
        assert isinstance(logic[1].operands[1], Field)
        assert isinstance(logic[1].operands[2], Field)
        assert str(logic[1]) == "( e OR a OR f )"
        assert len(logic[1]) == 3
        
        assert isinstance(logic[2], AND)
        assert isinstance(logic[2].operands[0], Field)
        assert isinstance(logic[2].operands[1], OR)
        assert isinstance(logic[2].operands[2], Field)
        assert isinstance(logic[2].operands[1].operands[0], Field)
        assert isinstance(logic[2].operands[1].operands[1], Field)
        assert str(logic[2]) == "( c AND ( d OR e ) AND a )"
        assert len(logic[2]) == 4
                
        assert isinstance(logic[3], OR)
        assert isinstance(logic[3].operands[0], AND)
        assert isinstance(logic[3].operands[1], Field)
        assert isinstance(logic[3].operands[0].operands[0], Field)
        assert isinstance(logic[3].operands[0].operands[1], Field)
        assert isinstance(logic[3].operands[0].operands[2], Field)
        assert str(logic[3]) == "( ( b AND c AND d ) OR e )" 
        assert len(logic[3]) == 4
        


from django.contrib.gis.forms import *
from django.contrib.gis.geos import MultiPoint, Point
from django.contrib.gis.measure import D
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _
import math


class CoordsField(CharField):
    def __init__(self, latitude_first=False, **kwargs):
        self.latitude_first = latitude_first
        kwargs['error_messages'] = {
            'invalid': _('Enter valid coordinate numbers.'),
            'no_pair': _('Enter a pair of numbers, separated by a comma.'),
            'out_of_range': _('Enter coordinates within range'),
        }
        super(CoordsField, self).__init__(**kwargs)
        
    def to_python(self, value):
        if not value:
            return None
        in_range = lambda lon,lat: (-90 < lat < 90) and (-180 < lon < 180)
        coords = value.split(',')
        if len(coords) != 2:
            raise ValidationError(self.error_messages['no_pair'])
        try:
            x,y = [float(each) for each in coords]
        except ValueError:
            raise ValidationError(self.error_messages['invalid'])
        else:
            if math.isnan( x ) or math.isnan( y ):
                raise ValidationError(self.error_messages['invalid'])
            elif self.latitude_first:
                if in_range(y, x):
                    return Point(y, x)
            elif in_range(x, y):
                return Point(x, y)
            raise ValidationError(self.error_messages['out_of_range'])
            

class BoundingBoxWidget(MultiWidget):
    def __init__(self, attrs=None):
        widgets = (CoordsField.widget, CoordsField.widget)
        super(BoundingBoxWidget, self).__init__(widgets, attrs)
    
    def decompress(self, value):
        if value is None:
            return [None, None]
        return [Point(*each) for each in value[0]]


class BoundingBoxField(MultiValueField):
    def __init__(self, latitude_first=False, **kwargs):
        self.latitude_first = latitude_first
        fields = (
            CoordsField(latitude_first=latitude_first),
            CoordsField(latitude_first=latitude_first),
        )
        super(BoundingBoxField, self).__init__(
            fields=fields, widget=BoundingBoxWidget, **kwargs)
        
    def compress(self, data_list):
        if len(data_list)==2:
            return MultiPoint(*data_list).envelope
        return None


class CircleField(CoordsField):
    def __init__(self, distance=5, unit='km', **kwargs):
        self.distance = D(**{unit: distance})
        super(CircleField, self).__init__(**kwargs)
        
    def to_python(self, value):
        if not value:
            return None
        point = super(CircleField, self).to_python(value)
        return (point, self.distance)



from django.core.exceptions import ValidationError
from django import forms
from .operators import BaseOperator, AND, OR


class QueryFormMixin(object):

    @property
    def validated_data(self):
        if not hasattr(self, '_validated_data'):
            self.set_parameters()
        return self._validated_data

    @property
    def parameters(self):
        if not hasattr(self, '_parameters'):
            self.set_parameters()
        return self._parameters
    
    def get_data(self, name):
        value = self.cleaned_data.get(name, None)
        return value if value else self._meta.defaults.get(name, None)
    
    def set_parameters(self):
        self._parameters = {}
        self._validated_data = {}
        for fieldname,lookup in self._meta.lookups.iteritems():
            value = self.get_data(fieldname)
            if value:
                self._validated_data[fieldname] = value
                self._parameters[lookup] = value
        for fields,call in self._meta.multifield_lookups.iteritems():
            values = []
            for fieldname in fields:
                value = self.get_data(fieldname)
                if value:
                    self._validated_data[fieldname] = value
                    self._parameters.pop(self._meta.lookups[fieldname])
                    values.append(value)
            if len(values) == len(fields):
                self._parameters.update(call(*values))
    
    def set_meta(self):                        
        if not hasattr(self, '_meta'):
            self._meta = type('Meta', (object,), {})()
        if not hasattr(self, 'Meta'):
            self.Meta = type('Meta', (object,), {})()
        self._meta.required = getattr(self.Meta, 'required', [])
        self.set_lookups_and_defaults()
        self.set_multifield_lookups()
        self.set_extralogic()
    
    def set_lookups_and_defaults(self):
        self._meta.defaults = {}
        self._meta.lookups = {}
        self._meta.no_defaults = getattr(self.Meta, 'no_defaults', False)
        for name,field in self.fields.iteritems():
             self._meta.lookups[name] = name
             if not self._meta.no_defaults:
                 if callable(field.initial):
                     self._meta.defaults[name] = field.initial()
                 else:
                     self._meta.defaults[name] = field.initial
        if hasattr(self.Meta, 'lookups'):
            for fieldname,alias in self.Meta.lookups.iteritems():
                self._meta.lookups[fieldname] = alias
    
    def set_multifield_lookups(self):
        self._meta.multifield_lookups = {}
        if hasattr(self.Meta, 'multifield_lookups'):
            for fields,alias in self.Meta.multifield_lookups.iteritems():
                self._meta.multifield_lookups[fields] = alias
    
    def set_extralogic(self):
        def get_logic(operator):
            args = []
            for value in operator.operands:
                if isinstance(value, (str, unicode)):
                    self.fields[value].attrname = value
                    args.append( self.fields[value] )
                elif isinstance(value, BaseOperator):
                    args.append( get_logic(value) )
            return operator.__class__.create(*args)
            
        self._meta.extralogic = []
        if hasattr(self.Meta, 'extralogic'):
            for each in self.Meta.extralogic:
                self._meta.extralogic.append( get_logic(each) )
                
    def clean_extralogic(self):
        for each in self._meta.extralogic:
            try:
                each.is_valid(self.validated_data, self.cleaned_data)
            except ValidationError as e:
                try:
                    self.add_error(e.subject, e.message)
                except AttributeError:
                    self.__errors[e.subject] = e.message



from django.http import HttpRequest
try:
    from django.http import JsonResponse
    native_json_response = True
except ImportError:
    from easy_response.http import JsonResponse
    native_json_response = False
from django.shortcuts import get_object_or_404
from django.forms import ModelForm
from functools import wraps


class validate(object):
    def __init__(self, form_class, add_instance_using='pk'):#, extra=None):
        self.form_class = form_class
        self.add_instance_using = add_instance_using
        # self.extra = extra
        
    def __call__(self, view):
        
        @wraps(view)
        def wrapper(*args, **kwargs):
            
            def get_form_kwargs(request):
                data = request.GET if request.method=='GET' else request.POST
                form_kwargs = {'data': data}
                if not hasattr(request, "FILES"): 
                    form_kwargs['files'] = request.FILES
                if issubclass(self.form_class, ModelForm) and kwargs:
                    value = kwargs.get(self.add_instance_using, None)
                    if value != None:
                        model = self.form_class.Meta.model
                        instance = get_object_or_404(model, pk=value)
                        form_kwargs['instance'] = instance    
                # if self.extra != None:
                #     form_kwargs.update(self.extra(request))
                return form_kwargs
            
            def error_response(form):
                if native_json_response:
                    return JsonResponse(form.errors, status=400, safe=False)
                else:
                    return JsonResponse(form.errors, status=400)
            
            def validate(request):
                request.form = self.form_class(**get_form_kwargs(each))
                if request.form.is_valid():
                    return view(*args, **kwargs)
                return error_response(request.form)
                
            for each in args:
                if isinstance(each, HttpRequest):
                    return validate( each )
            return validate( args[0] )        
        
        return wrapper
        